/* vc_cpp */

/* 
 * Problem  : 	DESKTOP
 * Contest  : 	UOI-2006 (Dnepropetrovsk)
 * Type     : 	Solution
 * Date     : 	March 30, 2006
 * Author   : 	Vladimir Tkachuk
 * Language :	C++
 * Compiler :	VC 6.0
 * Algorithm:	Best known solution
 */

#include<vector>
#include<algorithm>
#include<string>
#include<fstream>
#include<utility>
#include<iostream>
#include <cmath>
#include<queue>
#include <sstream>
using namespace std;
int N,F,S;
vector< int> x,y;
vector<string> name;
vector<vector<int> > G;
void ReadData(string filename ){
	ifstream in(filename.c_str());
	in>>N>>S>>F;
	x.assign(N,0);
	y.assign(N,0);
	--S;
	--F;
	name.assign(N,"");
	for (int i = 0; i<N; ++i){
		in >> x[i] >> y[i] >> name [i];
	}
	in.close();
}
void BuildNames(){ //Builds correspondence between names with O(NlogN)
	vector<pair<string,int> > GN(N);
	int i;
	for(i =0; i<N; ++i){
		GN[i] = make_pair(name[i],i);
	}
	int t;
	sort(GN.begin(),GN.end());
	for(i = 0; i<N; ++i){
		if (i==0||GN[i].first[0]!=GN[i-1].first[0]){
			t = i;
			for (int j = 0; j<N; ++j)
				if (name[j][0]!=GN[i].first[0])
					G[j].push_back(GN[i].second);
		}
		if (i<N-1&&GN[i].first[0]==GN[i+1].first[0]){
			G[GN[i].second].push_back(GN[i+1].second);
		}
		else
		if (t!=i){
			G[GN[i].second].push_back(GN[t].second);
		}
	}
}
int closer(double x1, double y1, double x2, double y2){
	double d1 = x1*x1+y1*y1;
	double d2 = x2*x2+y2*y2;
	if (d1-d2>0.5) return -1;
	if (d2-d1>0.5) return 1;
	return 0;
}
int FindUP(int ind){ //Find the nearlest "UP"
	int ans = -1;
	for(int i = 0; i< N; ++i){
		if (y[i]>y[ind]){
			int dy = y[i]-y[ind];
			int dx = x[i]-x[ind];
			if (abs(dx)>dy) continue;
			if (ans <0) {
				ans =i;
				continue;
			}
			int c = closer(dx,dy,x[ans]-x[ind],y[ans]-y[ind]);
			if (c>0||(c==0&&(x[i]<x[ans]||(x[i]==x[ans]&&y[i]<y[ans])))){
				ans = i;
			}
		}
	}
	return ans;
}
int FindDOWN(int ind){ //Find the nearlest "DOWN"
	int ans = -1;
	for(int i = 0; i< N; ++i){
		if (y[i]<y[ind]){
			int dy = y[ind]-y[i];
			int dx = x[i]-x[ind];
			if (abs(dx)>dy) continue;
			if (ans <0) {
				ans =i;
				continue;
			}
			int c = closer(dx,dy,x[ans]-x[ind],y[ans]-y[ind]);
			if (c>0||(c==0&&(x[i]<x[ans]||(x[i]==x[ans]&&y[i]<y[ans])))){
				ans = i;
			}
		}
	}
	return ans;
}
int FindRIGHT(int ind){ //Find the nearlest "RIGHT"
	int ans = -1;
	for(int i = 0; i< N; ++i){
		if (x[i]>x[ind]){
			int dy = y[i]-y[ind];
			int dx = x[i]-x[ind];
			if (abs(dy)>=dx) continue;
			if (ans <0) {
				ans =i;
				continue;
			}
			int c = closer(dx,dy,x[ans]-x[ind],y[ans]-y[ind]);
			if (c>0||(c==0&&(x[i]<x[ans]||(x[i]==x[ans]&&y[i]<y[ans])))){
				ans = i;
			}
		}
	}
	return ans;
}
int FindLEFT(int ind){ //Find the nearlest "LEFT"
	int ans = -1;
	for(int i = 0; i< N; ++i){
		if (x[i]<x[ind]){
			int dy = y[i]-y[ind];
			int dx = x[ind]-x[i];
			if (abs(dy)>=dx) continue;
			if (ans <0) {
				ans =i;
				continue;
			}
			int c = closer(dx,dy,x[ans]-x[ind],y[ans]-y[ind]);
			if (c>0||(c==0&&(x[i]<x[ans]||(x[i]==x[ans]&&y[i]<y[ans])))){
				ans = i;
			}
		}
	}
	return ans;
}
void BuildGraph(){
	G.assign(N,vector<int>());
	BuildNames();
}
int MinDist(){
	vector<int> p(N,-1);
	queue<int> w;
	p[S] = 0;
	w.push(S);
	while (p[F]<0){
		int s = w.front();
		for (int i = 0; i<G[s].size(); ++i){
			if (p[G[s][i]]<0){
				p[G[s][i]] = p[s]+1;
				w.push(G[s][i]);
			}
		}
		int l;
		l = FindUP(s);
		if (l>=0&&p[l]<0){
			p[l]=p[s]+1;
			w.push(l);
		}
		l = FindDOWN(s);
		if (l>=0&&p[l]<0){
			p[l]=p[s]+1;
			w.push(l);
		}
		l = FindLEFT(s);
		if (l>=0&&p[l]<0){
			p[l]=p[s]+1;
			w.push(l);
		}
		l = FindRIGHT(s);
		if (l>=0&&p[l]<0){
			p[l]=p[s]+1;
			w.push(l);
		}
		w.pop();
	}
	return p[F];
}
void OutAnswer(int ans, string filename){
	ofstream out(filename.c_str());
	out<<ans<<endl;
	out.close();
}
int main(){
	ReadData("desktop.dat");
	BuildGraph();
	OutAnswer(MinDist(),"desktop.sol");
	return 0;
}