/* vc_cpp */

/* 
 * Problem  : 	DESKTOP
 * Contest  : 	UOI-2006 (Dnepropetrovsk)
 * Type     : 	Solution
 * Date     : 	March 20, 2006
 * Author   : 	Olexiy Oryeshko
 * Language :	C++
 * Compiler :	VC 6.0
 * Algorithm:	Best known solution
 */

#include <map>
#include <set>
#include <sstream>
#include <iostream>
#include <queue>
#include <deque>
#include <vector>
#include <string>
#include <math.h>
#include <algorithm>
#include <numeric>
#include <fstream>
using namespace std;
#ifdef __GNUC__
typedef long long lint;
typedef unsigned long long ulint;
#else
typedef __int64 lint;
typedef unsigned __int64 ulint;
#endif



#define FOR(iter, bound) for(iter=0; (iter) < (int)(bound); iter++)
#define SFOR(iter, start, bound) for (int iter = (int)(start); iter < (int)(bound); iter++)
#define ALL(C) C.begin(), C.end()
#define VSORT(vec) sort(ALL(vec))
#define MP(a, b) make_pair(a, b)
#define MDD(a, b, c) MP(MP(a, b), c)

typedef vector<string> VS;
typedef vector<VS> VVS;
typedef vector<int> VI;
typedef vector<VI> VVI;
typedef vector<bool> VB;
typedef vector<VB> VVB;
typedef vector<double> VD;
typedef vector<vector<double> > VVD;
typedef set<int> SI;
typedef set<double> SD;
typedef pair<int, int> PII;
typedef pair<PII, int> DD;
typedef pair<PII, DD> RR;

/// Constraints for the problem

const int MAX_VERTICES = 10000;
const int MAX_NAME_LENGTH = 50;
const lint MAX_COORD = 1000000;
const lint MIN_COORD = 0;



inline lint myAbs(lint a) {
  return (a >= 0) ? a : -a;
}

int randInt(int r1, int r2);


using namespace std;

int ReadInt(FILE* in){
  char buf[100];
  fscanf(in, "%s", buf);
  return atoi(buf);
  }

void PrintLint(FILE* out, lint ans)
  {
  bool isneg = false;
  if (ans < 0) isneg = true, ans *= -1;
  if (ans == 0)
    {
    fprintf(out, "0\n");
    return; 
    }
  string str = "";
  while (ans) {
    str += ((int)(ans % 10) + '0');
    ans /= 10;
    }
reverse(ALL(str));
str += "\n";
if  (isneg) str = "-" + str;
fprintf(out, str.c_str());
  }



int N, n1, n2;
VS names;
vector<lint> x;
vector<lint> y;
VVB dist;


int sect(lint dx, lint dy)
  {
  if (myAbs(dx) <= myAbs(dy)) return (dy > 0) ? 1 : 3;
  return dx < 0 ? 0 : 2;
  }

#ifdef DEBUG
template <class C> void PVI(const vector<C>& v) {FOR(i, v.size()) cout << v[i] << " "; cout << "\n";};
#else
template <class C> void PVI(const vector<C>& v) {};
#endif 

bool read(FILE* in);

bool isCoordOk(lint w)   { return w >= MIN_COORD && w <= MAX_COORD;  }

/// Checks whether test file is valid.
bool check(FILE* in)
  {
  try 
    {
    read(in);
    }
  catch (...)
    {
    cout << "Wrong test\n";
    return false;
    }
  int i,j;
  FOR(i, N) 
    {
    if (names[i].length() <= 0 ||names[i].length() > MAX_NAME_LENGTH)
      {
      cout << "Text for shortcut " << i << " is too long\n";
      return false;
      }
    FOR(j, names[i].length()) if (names[i][j] < 'A' || names[i][j] > 'Z') 
      {
      cout << "Text for shortcut " << i << " must contain only uppercase letters ('A' - 'Z')\n";
      return false;
      }
    }
  FOR(i, N) if (!isCoordOk(x[i]) || !isCoordOk(y[i]))
    {
    cout << "Coordinates for shortcut " << i << " are out of bounds.\n";
    return false;
    }
  vector<pair<string, int> > pp;
  FOR(i, N) pp.push_back(MP(names[i], i));
  sort(ALL(pp));
  FOR(i, N - 1) if (pp[i].first == pp[i + 1].first)
    {
    cout << "Shortcuts " << pp[i].second << " and " << pp[i + 1].second << " must not contain identical text\n";
    return false;
    }
  vector<pair<PII, int> > cc;
  FOR(i, N) cc.push_back(MP(MP(x[i], y[i]), i));
  sort(ALL(cc));
  FOR(i, N - 1) if (cc[i].first == cc[i + 1].first)
    {
    cout << "Shortcuts " << cc[i].second << " and " << cc[i + 1].second << " must not be at the same position\n";
    return false;
    }
  return true;
  }



bool read(FILE* in)
  {
  fscanf(in, "%i %i %i", &N, &n1, &n2);
  if (N < 1 || N > MAX_VERTICES) 
    {
    cout << "Wrong N\n";
    return false;
    }
  if (n1 < 1 || n1 > N || n2 < 1 || n2 > N) 
    {
    cout << "Wrong shortcuts numbers\n";
    return false;
    }
  n1--; n2--;
  int aa, bb;
  char c[100];
  int i;
  FOR(i, N) fscanf(in, "%i %i %s", &aa, &bb, &c), names.push_back(string(c)), x.push_back(aa), y.push_back(bb);
  return true;
  }

main()  
  {
  if (false)
    {
    /// This code can generate tests
    srand(17);
//    generateRandomTest("17", 1235);
    return;
    }
  int i;
  FILE *out = fopen ("desktop.sol", "w+");
  FILE *in = fopen ("desktop.dat", "r");


  read(in);

  VVS a(26);
  map<string, int> dd;
  FOR(i, N) 
    {
    dd[names[i]] = i;
    a[names[i][0] - 'a'].push_back(names[i]);
    }

  VVI conn(N);
  int j;
  FOR(i, 26) 
    {
    sort(ALL(a[i]));
    int prev = 0;
    FOR(j, a[i].size()) 
      {
      int nxt = dd[a[i][j]];
      conn[prev].push_back(nxt);
      prev = nxt;
      }
    if (a[i].size() == 0) continue;
    int ind1 = dd[a[i][0]];
    int lst = dd[a[i][a[i].size() - 1]];
    conn[lst].push_back(ind1);
    }

  VVI memo(N, VI(4, -1));
  lint dl = 10000000;
  dl *= dl;
  vector<vector<lint> > dd1(N, vector<lint>(4, dl));
  FOR(i, N) FOR(j, i) 
    {
    lint d = (x[i] - x[j])*(x[i] - x[j]) + (y[i] - y[j])*(y[i] - y[j]);
    int a = sect(x[i] - x[j], y[i] - y[j]);
    int prev = memo[j][a];
    if (d < dd1[j][a] || 
      (d == dd1[j][a] && x[i] < x[prev]) ||
      (d == dd1[j][a] && x[i] == x[prev] && y[i] < y[prev]))
      {
      dd1[j][a] = d;
      memo[j][a] = i;
      }
    a += 2, a %= 4;
    prev = memo[i][a];
//    if (prev == -1 ||
     if (d < dd1[i][a] || 
      (d == dd1[i][a] && x[j] < x[prev]) ||
      (d == dd1[i][a] && x[j] == x[prev] && y[j] <= y[prev]))
      {
      dd1[i][a] = d;
      memo[i][a] = j;
      }
    }
  FOR(i, N) FOR(j, 4) if (memo[i][j] != -1) conn[i].push_back(memo[i][j]);
  FOR(i, N) PVI(conn[i]);
  VI time(N, 100000000);
  time[n1] = 0;
  queue<int> q;
  q.push(n1);
  while (q.size())
    {
    int v = q.front();
    q.pop();
    int t = time[v] + 1;
    FOR(i, conn[v].size())
      {
      int v1 = conn[v][i];
      if (time[v1] > t) time[v1] = t, q.push(v1);
      }
    }
  /// Prints both to file and std out.
  fprintf(out, "%i\n", time[n2]);
  cout << time[n2] << "\n";
  }
