/* vc_cpp */

/* 
 * Problem  : 	DIST
 * Contest  : 	UOI-2006 (Dnepropetrovsk)
 * Type     : 	Solution
 * Date     : 	April 1, 2006
 * Author   : 	Taras Galkovskyy
 * Language :	C++
 * Compiler :	VC 6.0
 * Algorithm:	Right solution, while not optimized enough
 */

#include <iostream>
#include <vector>
#include <fstream>
#include <algorithm>
#include <queue>
#include <math.h>
using namespace std;

#define sqr(x) ((x)*(x))
#define EPS 1e-9

struct Point{
	int x, y;
	Point () {}
	Point (int _x, int _y){x = _x;y = _y;};
};

inline double dist (Point a, Point b){	return sqrt((double)sqr(a.x - b.x) + sqr(a.y - b.y));};
bool is_right_triad (Point A, Point B){	return (A.x * B.y - B.x * A.y <= 0) ? true : false;}

double dist2point (Point begin, Point end, Point x)
{
	double a = end.y - begin.y,
		b = begin.x - end.x,
		c = end.x*begin.y - begin.x*end.y,
		m = sqrt(sqr(a) + sqr(b));
	a /= m;	b /= m;	c /= m;
	return fabs(a * x.x + b * x.y + c);
}

vector <Point> p;

struct angle_sort_el {int id;	double a, d;};
inline bool less_angle_sorter (angle_sort_el a, angle_sort_el b)
{
	if (fabs(a.a - b.a) < EPS) return (a.d < b.d) ? true : false;
	else return (a.a > b.a) ? true : false;
}
struct less_angle_sort_el {	inline bool operator() (angle_sort_el a, angle_sort_el b){ return less_angle_sorter(a, b); } };

vector <Point> convex_hull ()
{
	vector <Point> res;
	if (p.size() <= 3)
	{
		for (int i = 0; i < (int)p.size(); ++i)
			res.push_back(p[i]);
		return res;
	}

	int i,	lower_left = 0;

	for (i = 0; i < (int)p.size(); ++i)
		if (p[lower_left].y > p[i].y ||
			p[lower_left].y == p[i].y &&
			p[lower_left].x > p[i].x)
			lower_left = i;

	if (lower_left != 0) swap(p[lower_left], p[0]);

	priority_queue <angle_sort_el, vector <angle_sort_el>, less_angle_sort_el> ang;
	angle_sort_el temp;
	for (i = 1; i < (int)p.size(); ++i)
	{
		temp.id = i;
		temp.d = dist(p[i],p[0]);
		temp.a = atan2((double)p[i].y-p[0].y, (double)p[i].x-p[0].x);
		ang.push(temp);
	}

	vector <int> resid;
	res.push_back(p[0]); resid.push_back(0);
	res.push_back(p[ang.top().id]); resid.push_back(ang.top().id); ang.pop();
	while (ang.size())
	{
		i = ang.top().id; ang.pop();
		while (res.size() > 1 &&
			is_right_triad(
			Point(res[res.size()-1].x - res[res.size()-2].x,
			res[res.size()-1].y - res[res.size()-2].y),
			Point(p[i].x - res[res.size()-1].x,
			p[i].y - res[res.size()-1].y))
			)
		{
			res.pop_back();
			resid.pop_back();
		}
		res.push_back(p[i]); resid.push_back(i);
	}
	return res;
}

double longest_altitude (Point& A, Point& B, Point& C)
{
	int i, j;
	vector <Point> X = convex_hull();
	
	vector <angle_sort_el> a;
	double bestd=-1;
	for (i = 0; i < (int)p.size(); ++i)
	{
		a.clear();
		for (j = 0; j < (int)p.size(); ++j)
			if (i != j)
			{
				angle_sort_el temp;
				temp.a = atan2((double)p[j].y-p[i].y, (double)p[j].x-p[i].x);
				temp.d = dist(p[i], p[j]);
				temp.id = j;
				a.push_back(temp);
			}
			sort(a.begin(), a.end(), &less_angle_sorter);
			reverse(a.begin(), a.end());

			vector <angle_sort_el>::iterator it1 = a.begin();
			vector <Point>::iterator it2 = X.begin();
			double d = dist2point(p[i], p[it1->id], *it2),
				prev;
			do
			{
				prev = d;
				it2++; if (it2 == X.end()) it2 = X.begin();
				d = dist2point(p[i], p[it1->id], *it2);
			}
			while (d < prev);

			for (; it1 != a.end(); ++it1)
			{
				d = dist2point(p[i], p[it1->id], *it2);
				do
				{
					if (d > bestd)
					{
						A = p[i]; B = p[it1->id]; C = *it2;
						bestd = d;
					}
					prev = d;
					it2++; if (it2 == X.end()) it2 = X.begin();
					d = dist2point(p[i], p[it1->id], *it2);
				}
				while (d > prev);

				if (it2 == X.begin()) it2 = X.end();
				it2--;
			}
	}
	return bestd;
}

/*
			ALGORITHM N^3 -- prostoy variant

double longest_altitude2 (Point& A, Point& B, Point& C)
{
	int i, j, k;
	double bestd=-1, d;
	for (i = 0; i < (int)p.size(); ++i)
		for (j = 0; j < (int)p.size(); ++j)
			if (i != j)
				for (k = 0; k < (int)p.size(); ++k)
					if (i != k && j != k)
					{
						d = dist2point(p[i], p[j], p[k]);
						if (d > bestd)
						{
							A = p[i]; B = p[j]; C = p[k];
							bestd = d;
						}
					}
	return bestd;
}
*/
int main ()
{
	int n;
	Point x;
	ifstream ifs("dist.dat");
	for (ifs >> n; n; n--)
	{
		ifs >> x.x >> x.y;
		p.push_back(x);
	}

	Point a, b, c;
	double d;

	d = longest_altitude(a, b, c);
	FILE * fout = fopen("altitude.sol", "w");
	fprintf(fout,"%.10lf\n",d);
	fclose(fout);
	
	return 0;
}